/*++ daepres.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Determine if the local NQS daemon is present.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	October 3, 1985.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#if	BSD42 | BSD43 | ULTRIX
#include "nqs.h"			/* NQS definitions */
#include <sys/file.h>
#else
#if	SGI | SYS52 | SYS53 | UNICOS | UTS
#include "nqspacket.h"			/* NQS local message packets */
#else
BAD SYSTEM TYPE
#endif
#endif
#include "transactcc.h"			/* TCMx_ codes */

#if	BSD42 | BSD43 | ULTRIX
#else
#if	SGI | SYS52 | SYS53 | UNICOS | UTS
extern long inter();			/* Send message packet */
extern void interclear();		/* Clear packet contents */
#else
BAD SYSTEM TYPE
#endif
#endif

/*** daepres
 *
 *
 *	int daepres():
 *	Determine if the local NQS daemon is present.
 *
 *	Returns:
 *		1: if the local NQS daemon IS present.
 *		0: if the local NQS daemon is NOT present.
 */
#if	BSD42 | BSD43 | ULTRIX
int daepres (runfile)
register struct confd *runfile;		/* Locked exclusively if the NQS */
					/* daemon is running just fine; */
					/* Unlocked if NQS is not running, */
					/* or is shutting down */
{
	if (flock (runfile->fd, LOCK_SH | LOCK_NB) == -1) {
		/*
		 *  The local NQS daemon has the queues file locked
		 *  exclusively with an advisory lock, and is therefore
		 *  running.
		 */
		return (1);		/* Local NQS daemon is running */
	}
	/*
	 *  We were able to successfully apply a shared advisory lock to
	 *  the queues file.  Therefore, the local NQS daemon is either
	 *  shutdown, or in the process of shutting down.
	 *
	 *  Before we return, we must give up the shared advisory lock,
	 *  so that a future incarnation of the resurrected NQS daemon, can
	 *  successfully apply an advisory exclusive lock on the queues
	 *  file.
	 */
	flock (runfile->fd, LOCK_UN);	/* Release the lock */
	return (0);			/* Local NQS daemon is not running */
}
#else
#if	SGI | SYS52 | SYS53 | UNICOS | UTS
int daepres ()
{
	interclear();
	if (inter (PKT_SENSEDAEMON) == TCML_COMPLETE) return (1);
	return (0);			/* The NQS daemon is not running */
}
#else
BAD SYSTEM TYPE
#endif
#endif
