/*++ fetchgrdb.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	This module contains three functions:
 *
 *		fetchgrgid(),
 *		fetchgrnam(), and
 *		closegrdb()
 *
 *	that respectively return the group structure for the specified
 *	local group-id, return the group structure for the specified
 *	local group-name, and close the group-database opened by fetchgrgid()
 *	or fetchgrnam().
 *
 *	This module is provided as an "onion-skin layer" residing above
 *	the locally implemented group-database access functions.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	January 20, 1986.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include <grp.h>			/* Password file structures */


/*
 *	External functions.
 */
extern void endgrent();			/* Close the local group file */
extern struct group *getgrent();	/* Read next group file entry */
extern int strcmp();			/* String comparison */


/*** fetchgrgid
 *
 *
 *	struct group *fetchgrgid():
 *	Return struct group structure for the specified local group-id.
 *
 *	NOTE:	This function does not open and close the
 *		group-database on successive invocations.
 *		It is up to the caller to invoke closegrdb()
 *		when access is no longer required to the
 *		group-database.
 */
struct group *fetchgrgid (gid)
register int gid;			/* Group-id to be identified */
{
	return (getgrgid (gid));
}


/*** fetchgrnam
 *
 *
 *	struct group *fetchgrnam():
 *	Return struct group structure for the specified group-name.
 *
 *	NOTE:	This function does not open and close the
 *		group-database on successive invocations.
 *		It is up to the caller to invoke closegrdb()
 *		when access is no longer required to the
 *		group-database.
 */
struct group *fetchgrnam (name)
register char *name;
{
	return (getgrnam (name));
}


/*** closegrdb
 *
 *
 *	void closegrdb():
 *
 *	Close the local group-database opened by fetchgrgid(), or
 *	by fetchgrnam().
 */
void closegrdb()
{
	endgrent();			/* Close the local group file */
}
