/*++ filecopy.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Copy a file.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	August 12, 1985.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

extern int errno;			/* System call error number */

/*** filecopyentire
 *
 *
 *	int filecopyentire():
 *	Copy a file from one open file descriptor to another.
 *	Don't stop until EOF.
 *
 *	Returns:
 *	      >=0: if successful (number of bytes transferred);
 *	       -1: if an error occurs (in which case errno is set).
 *		   Errno will equal zero, if we simply ran out of
 *		   file space writing the copy.
 */
long filecopyentire (from, to)
register int from;			/* "From" file descriptor */
register int to;			/* "To" file descriptor */
{
	char buffer [8192];		/* Copy buffer */
	register int nread;		/* One time number of bytes read */
	register int nwritten;		/* One time number of bytes written */
	register long cumulative;	/* Sum of nwritten's */

	errno = 0;			/* So that file limit returns 0 errno */
	cumulative = 0;
	while ((nread = read (from, buffer, sizeof(buffer))) != 0) {
		if (nread < 0) return (-1L);
		if ((nwritten = write (to, buffer, nread)) != nread) {
			printf ("E$Run out of file space: ");
			printf ("nread=%d, nwritten=%d, errno=%d\n",
				nread, nwritten, errno);
			return(-1L);
		}
		cumulative += nwritten;
	}
	return (cumulative);
}


/*** filecopysized
 *
 *
 *	int filecopysized():
 *	Copy a file from one open file descriptor to another.
 *	Stop as soon as the specified number of bytes have been moved.
 *
 *	Returns:
 *	      >=0: if successful, or if it reached a premature EOF
 *		   (number of bytes transferred);
 *	       -1: if an error occurs (in which case errno is set).
 *		   Errno will equal zero, if we simply ran out of
 *		   file space writing the copy.
 */
long filecopysized (from, to, bytes)
register int from;			/* "From" file descriptor */
register int to;			/* "To" file descriptor */
unsigned int bytes;			/* How many bytes to move */
{
	char buffer [8192];		/* Copy buffer */
	register int nread;		/* One time number of bytes read */
	register int nwritten;		/* One time number of bytes written */
	register int need;		/* Number of bytes still needed */

	errno = 0;			/* So that file limit returns 0 errno */
	for (need = bytes; need > 0; need -= nwritten) {
		if ((nread = read (from, buffer,
			need < sizeof(buffer) ? need : sizeof(buffer))) != 0) {
			if (nread < 0) return (-1L);
			if ((nwritten = write (to, buffer, nread)) != nread)
				return (-1L);
		}
		else return (bytes - need);
	}
	return (bytes);
}
