/*++ nqssleep.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	NQS version of sleep().  The System V one has a very nasty
 *	bug such that when a non-SIGALRM signal arrives which is to
 *	be caught, AND the SIGALRM signal just happens to arrive
 *	while handling the first signal, then we never return to
 *	the original signal handler.
 *
 *	Thank you again AT&T.  The mighty mountain has labored, and
 *	once again failed to even produce a mouse correctly.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	August 12, 1985.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include <signal.h>

/*
 *
 *
 *	Variables local to this module.
 */
static short alarm_semaphore;		/* SIGALRM semaphore */


/*** nqssleep
 *
 *
 *	nqssleep():
 *	Sleep for N seconds.
 */
void nqssleep (timetosleep)
int timetosleep;
{
	void alarmsig();		/* Catch SIGALRM */
	void (*sigalrm_sav)();		/* Save old SIGALRM handler */
	unsigned origalarm;		/* Original alarm time */

	origalarm = alarm (0);		/* Get original alarm set */
	sigalrm_sav = (void (*)())signal (SIGALRM, alarmsig); /* Get/Set SIGALRM handler */
	alarm_semaphore = 0;		/* Clear SIGALRM semaphore */
	alarm (timetosleep);		/* Set the alarm */
	while (alarm_semaphore == 0) pause();
	signal (SIGALRM, sigalrm_sav);	/* Restore SIGALRM state */
	alarm (origalarm);		/* Restore the original alarm */
}


/*** alarmsig
 *
 *
 *	void alarmsig():
 *
 *	This function serves as the signal catcher for SIGARLM signals
 *	occuring as the result of an alarm () call in the return_file()
 *	function above.
 *
 *	This function SETS the "alarm_semphore" flag.
 */
static void alarmsig ()
{
	signal (SIGALRM, alarmsig);
	alarm_semaphore++;		/* SIGALRM received */
}
