/*++ sholbymach.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Print on stdout the limits that are meaningful for a machine.
 *
 *
 *	Author:
 *	-------
 *	Robert W. Sandstrom, Sterling Software Incorporated.
 *	January 13, 1986.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include <stdio.h>			/* for stderr */
#include "nqs.h"			/* NQS types and definitions */
#include NETDB				/* Network database header file; */
					/* Proto/Makefile will tell us */
					/* where to find netdb.h */

extern struct hostent *gethostbyname();	/* Get hostent structure for name */
extern int localmid();			/* Get local machine-id */
extern int nmap_get_mid();		/* Get machine-id for name */
extern int shoalllim();			/* Show all (local) limits */


/*** sholbymach
 *
 *
 *	int sholbymach():
 *
 *	Print on stdout the limits that are meaningful
 *	for the named machine.
 *
 *	Returns: 0 if output was produced.
 *		-1 if no output was produced.
 */
int sholbymach (paramfile, itsname, flags)
struct confd *paramfile;		/* This pointer useful for */
					/* local case only */
char *itsname;				/* The machine in question */
long flags;				/* SHO_? */
{
	mid_t itsmid;
	mid_t mymid;
	struct hostent *ent;		/* Host table entry structure */

	if ((ent = gethostbyname (itsname)) == (struct hostent *) 0) {
		fprintf (stderr, "Invalid hostname specification: ");
		fprintf (stderr, "%s.\n", itsname);
		return (0);
	}
	switch (nmap_get_mid (ent, &itsmid)) {
	case NMAP_SUCCESS:		/* Successfully got machine-id */
		break;
	case NMAP_ENOMAP:		/* What?  No mid defined! */
		fprintf (stderr, "Invalid hostname specification: ");
		fprintf (stderr, "%s.\n", itsname);
		return (0);
	case NMAP_ENOPRIV:		/* No privilege */
		fprintf (stderr, "Unexpected error in nmap_get_mid(), ");
		fprintf (stderr, "machine = %s.\n", itsname);
		return (0);
	default:
		fprintf (stderr, "Unexpected error in nmap_get_mid(), ");
		fprintf (stderr, "machine = %s.\n", itsname);
		return (0);
	}
	if (localmid (&mymid) != 0) {
		fprintf (stderr, "Unexpected error in localmid().\n");
		return (0);
	}
	if (itsmid == mymid) {
		if (telldb (paramfile) != -1) {	/* if we have read before, */
			seekdb (paramfile, 0L);	/* seek to beginning */
		}
		return (shoalllim (paramfile, flags));
	}
	else {
		fprintf (stderr, "You are asking about a remote machine.\n");
		fprintf (stderr, "Networking is not supported in this ");
		fprintf (stderr, "implementation.\n");
		return (0);
	}
}
