/*++ nmapdb.h - nmap database internal structure
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 * STANDARDS VIOLATIONS:
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */
/*
 *	Configurable #defs that can be tailored for an individual system.
 */
#define	MAX_PATHLENGTH	1024	/* The maximum length of a file pathname */

#define	MAP_DIRECTORY	"/etc/nmap"
				/* Map database file directory.  ALL nmap */
				/* files are placed here.  This directory */
				/* should be created by the cmd/nmap */
				/* makefile.  The nmap_create() */
				/* function does NOT automatically create */
				/* this directory. */

#define MACHINES	"machines"
				/* Name of machine to mid mapping file in */
				/* the directory:  MAP_DIRECTORY. */

#define	MAX_NAMELENGTH	255	/* Maximum length of a machine name */
				/* The maximum length of a machine name */
				/* plus the associated structure overhead */
				/* (see name_map struct defn below) must */
				/* NOT exceed ATOMICBLKSIZ bytes in length! */

#define	ALIGNTYPE	double	/* Alignment type */

#define	HASHNAMENTS	256	/* Number of machine-name to mid hash chains */

#define	HASHENTSPERBLK	(ATOMICBLKSIZ / sizeof (long))
				/* Number of entries per block for a hash */
				/* table. */

#define	HASHNAMBLKS	((HASHNAMENTS - 1) / HASHENTSPERBLK + 1)
				/* Number of blocks that must be allocated */
				/* for the machine-name to mid hash table */

#define	MAX_MIDS	100	/* Maximum number of machine id's in the */
				/* network. Used by nmap_show (). */

/*
 *	WARNING:
 *
 *	These relations may have to be changed for some compiler/machine
 *	implementations based upon any padding algorithms the compiler
 *	follows when declaring structures.
 */
#define	GIDHDRSIZ	(sizeof (struct gidmap8) - 8 * sizeof (gid_t))

#define	UIDHDRSIZ	(sizeof (struct uidmap8) - 8 * sizeof (uid_t))

#define	GIDSPERBLK	((ATOMICBLKSIZ - GIDHDRSIZ) / (2 * sizeof (gid_t)))
				/* The number of group-id mappings per block */

#define	UIDSPERBLK	((ATOMICBLKSIZ - UIDHDRSIZ) / (2 * sizeof (uid_t)))
				/* The number of user-id mappings per block */

struct xidparm {
	union {
		struct {
			uid_t remote_uid;
			uid_t local_uid;
		} u;
		struct {
			gid_t remote_gid;
			gid_t local_gid;
		} g;
	} u;
};


/* Data structures for database */

struct m_header {
	char name [MAX_NAMELENGTH+1];
				/* Null terminated machine name.	   */
	char defuid_enabled;	/* Default user-id mapping enabled.	   */
	char defgid_enabled;	/* Default group-id mapping enabled.	   */
	uid_t defuid;		/* Default user-id mapping value.	   */
	gid_t defgid;		/* Default group-id mapping value.	   */
	time_t enatime;		/* When to reenable mapping, 0=never, 1=on */
};

struct gidmap8 {		/* Only used for GIDSPERBLK computation	*/
	long next;		/* Addr of next block in hash chain	*/
	unsigned n_pairs;	/* There are this many gid to gid map	*/
				/* pairs in this block.			*/
	gid_t pairs [8];	/* Gid to gid pairs......		*/
};

struct gidmap {
	long next;		/* Addr of next block in hash chain	*/
	unsigned n_pairs;	/* There are this many gid to gid map	*/
				/* pairs in this block.			*/
	gid_t pairs [GIDSPERBLK];
};				/* Gid to gid pairs......		*/

struct uidmap8 {		/* Only used for UIDSPERBLK computation	*/
	long next;		/* Addr of next block in hash chain	*/
	unsigned n_pairs;	/* There are this many uid to uid map	*/
				/* pairs in this block.			*/
	uid_t pairs [8];	/* Uid to uid pairs......		*/
};

struct uidmap {
	long next;		/* Addr of next block in hash chain	*/
	unsigned n_pairs;	/* There are this many uid to uid map	*/
				/* pairs in this block.			*/
	uid_t pairs [UIDSPERBLK];
};				/* Uid to uid pairs......		*/

struct hashblk {
	long chain [HASHENTSPERBLK];	/* Hash chain pointers		*/
};

struct nammap {
	mid_t	 mid;		/* Machine-id				*/
	unsigned short size;	/* Number of bytes contained in this	*/
				/* entry (even if NOT allocated!)	*/
	unsigned short namelen;	/* Length of machine-name.		*/
				/* Zero implies this entire entry is	*/
				/* not used at all.			*/
	char	 name [MAX_NAMELENGTH];
};				/* Name of machine.			*/

struct n_header {
	long nexthash;		/* Address of next block in hash chain	*/
	struct nammap map;	/* First mapping entry in the block	*/
};

struct rawblock {		/* Raw map file block properly aligned	*/
	union {
		ALIGNTYPE alignment;		/* Align the block	*/
		struct m_header mhdr;		/* Machine file hdr blk	*/
		struct uidmap umap;		/* User-id mapping blk	*/
		struct gidmap gmap;		/* Group-id mapping blk	*/
		struct hashblk hash;		/* Hash ptr block	*/
		struct n_header nhdr;		/* First nammap entry	*/
		char bytes [ATOMICBLKSIZ];	/* Raw block contents	*/
	} u;
};
