/*++ nqs_enfile.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Determine if the last file open error operation was caused
 *	by a lack of file descriptors, and if so--display the proper
 *	error output.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	August 12, 1985.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include <stdio.h>
#include <errno.h>
#include "nqs.h"
#include "nqsxvars.h"			/* Global vars */

extern int errno;			/* Error number */

/*** nqs_enfile
 *
 *
 *	int nqs_enfile():
 *	Diagnose a shortage of file descriptors.
 *
 *	Returns:
 *		0: if errno is NOT either EMFILE or ENFILE.
 *		1: if errno is either EMFILE or ENFILE in which
 *		   case error output is generated.
 */
int nqs_enfile ()
{
	if (errno == ENFILE || errno == EMFILE) {
		if (errno == ENFILE) {
			printf ("E$Unable to open file.\n");
			printf ("I$System file table full.\n");
			fflush (stdout);
			return (1);
		}
		printf ("F$Unable to open file.\n");
		printf ("I$Max open files per-process limit reached.\n");
		fflush (stdout);
		return (1);
	}
	return (0);		/* Nope, not ENFILE or EMFILE. */
}
