/*++ nqs_modreq.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Modify a queued NQS request.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	October 8, 1985.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include <stdio.h>
#if	UNICOS
#include <fcntl.h>
#include <sys/category.h>
#include <sys/resource.h>
#else
#if	SGI | SYS52 | SYS53 | UTS
#include <fcntl.h>
#else
#if	BSD42 | BSD43 | ULTRIX
#include <sys/file.h>
#else
BAD SYSTEM TYPE
#endif
#endif
#endif
#include "nqs.h"			/* NQS constants and data types */
#include "transactcc.h"			/* Transaction completion codes */
#include "nqsxvars.h"			/* Global vars */

extern int errno;
extern struct request *nqs_fndreq();	/* Find request by sequence# & mid */
extern void pack6name();		/* Compute name on 6-bit alphabet */
extern int readreq();			/* Read the request header portion */
					/* of a control file for an NQS req */
extern int writereq();			/* Write/rewrite request header */
extern long limit();
extern void a2s_a2aset();		/* Add to arriving set */
extern void a2s_a2hset();		/* Add to hold set in queue */
extern void a2s_a2qset();		/* Add to queued set in queue */
extern void a2s_a2wset();		/* Add to wait set in queue */
extern void udb_qorder();		/* Update request ordering file */

/*** nqs_modreq
 *
 *
 *	long nqs_modreq():
 *	Modify a queued NQS request.
 *
 *	Returns:
 *		TCML_COMPLETE:	 if the request was successfully
 *				 modified.
 *		TCML_NOSUCHREQ:	 if the specified request does not
 *				 exist on this machine.
 *		TCML_NOTREQOWN:	 if the mapped user-id does not
 *				 match the current mapped user-id
 *				 of the request owner.
 *		TCML_REQRUNNING: if the specified request is running.
 */
long nqs_modreq (mapped_uid, orig_seqno, orig_mid, modtype, param1, param2, inf)
uid_t mapped_uid;			/* Mapped owner user-id */
long orig_seqno;			/* Original request sequence# */
mid_t orig_mid;				/* Original machine-id of req */
long modtype;				/* Type of modification */
long param1;				/* First modification parameter */
long param2;				/* Second modification parameter */
int inf;				/* Set to infinite flag */
{
	struct request *predecessor;	/* Predecessor in req set in queue */
	int state;			/* Request queue state: RQS_ */
	struct request *req;		/* Ptr to request structure for */
					/* located req, if found */
	char path[MAX_PATHNAME+1];	/* pathname of control file */
	int cfd;			/* file descriptor for control file */
	struct rawreq rawreq;		/* raw request structure */
	long jid;			/* UNICOS job ID */
	long lim;			/* New limit */
	void (*a2s_func)();		/* Function to add to request set */

	/*
	 *  Locate the request.
	 */
	state = RQS_STAGING | RQS_QUEUED | RQS_WAITING | RQS_HOLDING
		| RQS_ARRIVING | RQS_RUNNING;

	predecessor = (struct request *) 0;
	if ((req = nqs_fndreq (orig_seqno, orig_mid, &predecessor,
			       &state)) == (struct request *) 0) {
		/*
		 *  The request was not found in any of the local queues.
		 */
		return (TCML_NOSUCHREQ);
	}
	if (mapped_uid && req->v1.req.uid != mapped_uid) {
		/*
		 *  This request cannot be affected by the client.
		 */
		return (TCML_NOTREQOWN);
	}
#if	UNICOS
#else
#if	SGI | SYS52 | SYS53 | UTS | BSD42 | BSD43 | ULTRIX
	if (state == RQS_RUNNING) {
		return (TCML_REQRUNNING);
	}
#else
BAD SYSTEM TYPE
#endif
#endif

	/*
	 *  Make the requested modification.
	 */
	pack6name (path, Nqs_control,
		  (int) (orig_seqno % MAX_CTRLSUBDIRS), (char *) 0,
		  (long) orig_seqno, 5, (long) orig_mid, 6, 0, 0);
	if ((cfd = open( path, O_RDWR )) < 0) {
		printf("E$nqs_mov(): Unable to open control file (%s).", path);
		printf("  Errno = %d\n", errno);
		fflush(stdout);
		return (TCML_UNAFAILURE);
	}
	if (readreq(cfd, &rawreq) < 0) {
		close(cfd);
		printf("E$nqs_mov(): Read of request header failed.");
		printf("  Errno = %d\n", errno);
		return (TCML_UNAFAILURE);
	}

	/*
	 *  Set the new limits.
	 */
	rawreq.v.bat.explicit |= modtype;
	if (inf) {				/* if infinite flag set */
		rawreq.v.bat.infinite |= modtype;
		switch (modtype) {		/* Change per-process limits */
		case LIM_PRMEM:
			rawreq.v.bat.infinite |= LIM_PPMEM;
			break;
		case LIM_PRCPUT:
			rawreq.v.bat.infinite |= LIM_PPCPUT;
			break;
		}
	} else {
		rawreq.v.bat.infinite &= ~modtype;
		switch (modtype) {
		case LIM_PRMEM:
			rawreq.v.bat.prmemsize.max_quota = param1;
			rawreq.v.bat.prmemsize.warn_quota = param1;
			rawreq.v.bat.prmemsize.max_units = param2;
			rawreq.v.bat.prmemsize.warn_units = param2;
			rawreq.v.bat.explicit |= LIM_PPMEM;
			rawreq.v.bat.infinite &= ~LIM_PPMEM;
			rawreq.v.bat.ppmemsize.max_quota = param1;
			rawreq.v.bat.ppmemsize.warn_quota = param1;
			rawreq.v.bat.ppmemsize.max_units = param2;
			rawreq.v.bat.ppmemsize.warn_units = param2;
			break;
		case LIM_PPNICE:
			lim = rawreq.v.bat.ppnice;	/* Get previous value */
			rawreq.v.bat.ppnice = param1;
			break;
		case LIM_PRCPUT:
			rawreq.v.bat.prcputime.max_seconds = param1;
			rawreq.v.bat.prcputime.warn_seconds = param1;
			rawreq.v.bat.prcputime.max_ms = param2;
			rawreq.v.bat.prcputime.warn_ms = param2;
			rawreq.v.bat.explicit |= LIM_PPCPUT;
			rawreq.v.bat.infinite &= ~LIM_PPCPUT;
			rawreq.v.bat.ppcputime.max_seconds = param1;
			rawreq.v.bat.ppcputime.warn_seconds = param1;
			rawreq.v.bat.ppcputime.max_ms = param2;
			rawreq.v.bat.ppcputime.warn_ms = param2;
			break;
		case LIM_PRIORITY:
			rawreq.rpriority = param1;
			req->v1.req.priority = param1;
			nqs_deque (req); 	/* Remove request from set */
			if (state & RQS_ARRIVING) {
				a2s_func = a2s_a2aset;
			} else if (state & RQS_QUEUED) {
				a2s_func = a2s_a2qset;
			} else if (state & RQS_WAITING) {
				if (param1 == MAX_RPRIORITY &&
				    req->queue->waitset != (struct request *)0)
					/*
					 *  Force this request to be at the
					 *  first position in the waiting list
					 */ 
					req->v1.req.state_time =
						req->start_time = 0;
				if (param1 == 0 && 	/* MIN_RPRIORITY */
				    req->queue->waitset != (struct request *)0)
					/*
					 *  Force this request to be at the
					 *  last position in the waiting list
					 */
					req->v1.req.state_time =
						req->start_time = 0x7fffffff;
				a2s_func = a2s_a2wset;
			} else if (state & RQS_HOLDING) {
				a2s_func = a2s_a2hset;
			}
			a2s_func (req, req->queue); 	/* Requeue request  */
			udb_qorder (req->queue);	/* and update queue */
			break;
		default:
			close(cfd);
			return (TCML_NOSUCHQUO);
		}
	}
	if (writereq(cfd, &rawreq) < 0) {
		printf("E$nqs_mov(): Write of request header failed.");
		printf("  Errno = %d\n", errno);
		close(cfd);
		return (TCML_UNAFAILURE);
	}
	close(cfd);

#if	UNICOS
	if (state != RQS_RUNNING) {
		return (TCML_COMPLETE);
	}
	jid = Runvars [req->reqindex].process_family;
	switch (modtype) {
	case LIM_PRMEM:
		if (inf) {
			lim = 0;
		} else {
			switch (param2) {
			case QLM_BYTES:
				lim = param1 / MEM_LIMIT_GRAN;
				break;
			case QLM_WORDS:
				lim = (param1 * BYTES_PER_WORD) /
					MEM_LIMIT_GRAN;
				break;
			case QLM_KBYTES:
				lim = (param1 * 0x400) / MEM_LIMIT_GRAN;
				break;
			case QLM_KWORDS:
				lim = (param1 * BYTES_PER_WORD * 0x400) /
					MEM_LIMIT_GRAN;
				break;
			case QLM_MBYTES:
				lim = (param1 * 0x100000) / MEM_LIMIT_GRAN;
				break;
			case QLM_MWORDS:
				lim = (param1 * BYTES_PER_WORD * 0x100000) /
					MEM_LIMIT_GRAN;
				break;
			case QLM_GBYTES:
				lim = (param1 * 0x40000000) / MEM_LIMIT_GRAN;
				break;
			case QLM_GWORDS:
				lim = ((param1 * 0x40000000) / MEM_LIMIT_GRAN) *
					BYTES_PER_WORD;
				break;
			}
		}
		limit(C_JOB, jid, L_MEM, lim);
		break;
	case LIM_PPNICE:
		nicem (C_JOB, jid, param1 - lim);
		break;
	case LIM_PRCPUT:
		if (inf) {
			lim = 0;
		} else {
			lim = param1 * CPU_LIMIT_GRAN;
		}
		limit (C_JOB, jid, L_CPU, lim);
		break;
	}
#endif
	return (TCML_COMPLETE);
}
