/*++ nqs_wakdes.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Re-enable pipe queue destinations that are presently in retry
 *	mode, whose retry wait time has elapsed.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	May 30, 1986.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include "nqs.h"
#include "nqsxvars.h"			/* Global vars and directories */

extern void nqs_vtimer();		/* Set virtual timer */
extern void psc_spawn();		/* Maybe spawn some pipe reqs */
extern time_t time();			/* GMT time since 0:00:00 Jan 1, 1970 */
extern void udb_destination();		/* Update destination */

/*** nqs_wakdes
 *
 *
 *	void nqs_wakdes():
 *
 *	Re-enable pipe queue destinations that are presently in retry
 *	mode, whose retry wait time has elapsed.
 */
void nqs_wakdes()
{
	register struct pipeto *dest;	/* Pipe queue destination */
	time_t timenow;			/* Current GMT time */

	time (&timenow);		/* Get current time */
	dest = Pipetoset;		/* Global pipe queue destination set */
	while (dest != (struct pipeto *) 0) {
		if (dest->status == DEST_RETRY) {
			/*
			 *  This destination is in the retry state, and
			 *  has NOT been re-enabled.
			 */
			if (dest->retry_at > timenow) {
				/*
				 *  Set alarm for destination retry in
				 *  the future.
				 */
				nqs_vtimer (&dest->retry_at, nqs_wakdes);
			}
			else {
				/*
				 *  This destination must be enabled
				 *  (leaving the retry bit set)!
				 */
				dest->status |= DEST_ENABLED;
				udb_destination (dest);	/* Update database */
			}
		}
		dest = dest->next;	/* Examine next destination */
	}
	/*
	 *  Maybe spawn some pipe queue requests, and update and
	 *  pipe queue request-ordering files that need to be
	 *  modified.
	 */
	psc_spawn();			/* Maybe spawn some pipe requests */
}
