/*++ qdev.c - Network Deviceing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Display device status information.
 *	This program must run as a setuid root program.
 *
 *
 * RETURNS:
 *	
 * 	0	-  if output produced
 *	1	-  if an error occurred. A message is sent to the standard
 *			output file for every error.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	October 24, 1985.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 */

#include <stdio.h>
#if	SGI | SYS52 | SYS53 | UNICOS | UTS
#include <fcntl.h>			/* File control */
#else
#if	BSD42 | BSD43 | ULTRIX
#include <sys/file.h>
#else
BAD SYSTEM TYPE
#endif
#endif
#include "nqs.h"
#include "nqsdirs.h"			/* NQS files and directories */


/*
 *	External functions:
 */
extern void bufstdout();		/* Block buffer stdout */
extern struct confd *opendb();		/* Open an NQS database file */
extern int shoalldev();			/* Show all devices */
extern int shodbyname();		/* Show a device given by name */

/*
 *	Global variables:
 */
char *Qdev_prefix = "Qdev";

/*** main
 *
 *
 *	qdev [ <device-name(s)> ]
 */
main (argc, argv)
int argc;
char *argv[];				/* Possible list of devices */
{
	struct confd *devicefile;
	int exitcode = 0;			/* Function exit code */

	if (chdir (Nqs_root) == -1) {
		fprintf (stderr, "%s(FATAL): Unable to chdir() to the NQS ",
			 Qdev_prefix);
		fprintf (stderr, "root directory.\n");
		exit (1);
	}
	if ((devicefile = opendb (Nqs_devices, O_RDONLY)) == NULL) {
		fprintf (stderr, "%s(FATAL): Unable to open the NQS device ",
			 Qdev_prefix);
		fprintf (stderr, "definition database file.\n");
		exit (1);
	}
	/*
	 *  Block buffer stdout for efficiency.
	 */
	bufstdout();
	if (*++argv == NULL) exitcode |= shoalldev (devicefile);
	else {
		while (*argv != NULL) {		/* Show info on a device */
			exitcode |= shodbyname (devicefile, *argv);
			argv++;
		}
	}
	/*
	 *  Flush output buffers and exit.
	 */
	fflush (stdout);
	fflush (stderr);
	exit (exitcode);
}
