/*++ qjob.c - Network Queueing System
 *
 * $Source$
 *
 * DESCRIPTION:
 *
 *	Display queue status information in a networked environment.
 *	This program must run as a setuid program.
 *
 *
 *	Author:
 *	-------
 *	Christian Boissat, CERN, Geneva, Switzerland.
 *	August 27, 1991.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision$ $Date$ $State$)
 * $Log$
 *
 *	January 27, 1992	Add option -a showing all requests
 *
 */
#include <stdio.h>
#include "nqs.h"			/* NQS types and definitions */
#include "nqsdirs.h"			/* NQS files and directories */
extern char *strpbrk();			/* Locate character in string */

#define BSERVERS "/etc/batchservers"


/*** main
 *
 *	qjob [ -a ]
 *
 */
main(argc, argv)
int argc;
char *argv[];
{
	FILE * bservfile;
	char srvname[4096];
	long flags;			/* SHO_??? */
	char *p;
	char *argument;
	char *args = (char *)0;

	if ((bservfile = fopen (BSERVERS, "r")) == (FILE *)0) {
		fprintf (stderr, "No batch server configured\n");
		exit (1);
	}
	flags = SHO_BATCH | SHO_DEVICE | SHO_PIPE | SHO_REQ;
	while (*++argv != (char *)0 && **argv == '-') {
		argument = *argv;
		while (*++argument != '\0') {
			switch (*argument) {
			case 'a':
				flags |= SHO_R_ALLUID;
				break;
			default:
				fprintf (stderr, "Invalid option flag ");
				fprintf (stderr, "specified.\n");
				fprintf (stderr, "Command format is:\n\n");
				fprintf (stderr, "    qjob [-a]\n\n");
				exit (1);
			}
		}
	}
	flags |= (SHO_RS_EXIT | SHO_RS_RUN | SHO_RS_QUEUED | SHO_RS_HOLD);
	/*
	 *  Firstly look after local host
	 */
	main_dsp (0, &args, flags, (char *)0, (char *)0, (char *)0);
	/*
	 *  Get central server names
	 */
	while (fgets (srvname, sizeof(srvname), bservfile) != (char *)0) {
		if ((p = strpbrk (srvname, " \t\n")) != (char *) 0) *p = '\0';
		main_dsp (0, &args, flags, (char *)0, (char *)0, srvname);
	}
	fclose (bservfile);
	printf ("\n");
	exit (0);
}
